string Description = "This shader will use a camera lighting effect making the object shading look the same from any direction";
string Thumbnail = "Constant Lighting.png";

float4x4 WorldViewProj : WorldViewProjection;
float4x4 World : World;
float4x4 WorldView : WorldView;
float4 eyePos : CameraPosition;

float Brightness
<
	string UIWidget = "slider";
	float UIMax = 1.0;
	float UIMin = 0.0;
	float UIStep = 0.01;
> = 0.333000;

struct app_in
{
	float4 pos : POSITION;
	float3 normal : NORMAL;
};

struct vs_out
{
	float4 pos : POSITION;
	float angle : TEXCOORD0;
};

vs_out DefaultVertexShader( app_in IN )
{
	vs_out OUT;
	
	OUT.pos = mul( IN.pos, WorldViewProj );
	
	float4 wPos = mul( IN.pos, World );
	float3 wNormal = mul( IN.normal, (float3x3)World );
	
	float angle = saturate( dot( normalize(wNormal), normalize(eyePos.xyz - wPos.xyz) ) );
	OUT.angle = angle / 1.5;
	
	return OUT;
}

float4 DefaultPixelShader( vs_out IN ) : COLOR
{
	float angle = saturate(IN.angle) + Brightness;
	
	float4 color = float4(0.5,1.2,2.2,0)*angle;
	
	return float4(color.xyz,1);
}

technique MyShader
{
	pass p0
	{
		VertexShader = compile vs_1_1 DefaultVertexShader( );
		PixelShader = compile ps_1_1 DefaultPixelShader( );
	}
}
