//Shader code by Mark Blosser
//WEBSITE:  www.hyrumark.com
//EMAIL:  mark@hyrumark.com

//Description:  Shader for dynamic entities, normal mapping,specular, reflectivity,
// and "hero" spec that always faces player.  "LightSource" variable is pulled from FPSC for accurate lighting.
//
//Textures: 
//"texture_D.dds"      diffuse texture
//"texture_I.dd"       specular texture, alpha channel is reflectivity mask
//"texture_S.dds"       reflectivity texture
// Modifed by Uzi Idiot
/************* UNTWEAKABLES **************/

float4x4 World      : WORLD;

float4x4 WorldViewProj : WorldViewProjection;
float4x4 WorldIT : WorldInverseTranspose;
float4x4 ViewInv : ViewInverse;
float4x4 View : View;
float4 eyePos : CameraPosition;
float4 clipPlane : ClipPlane;



/******TWEAKABLES***************************/

float SpecExpon : Power
<
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName =  "specular power";
> = 64.0;


/******VALUES PULLED FROM FPSC - NON TWEAKABLE**********/

float4 AmbiColor : Ambient
<
    string UIName =  "Ambient Light Color";
> = {0.1f, 0.1f, 0.1f, 1.0f};

float4 SurfColor : Diffuse
<
    string UIName =  "Surface Color";
    string UIType = "Color";
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 LightSource
<
    string UIType = "Fixed Light Source";
> = {500000.0f,50000.0f, -0.0f, 1.0f};

//----------------
// FOG sensitivity
//----------------
float4 FogColor : Diffuse
<
    string UIName =  "Fog Color";
    string UIType = "Color";
> = {0.0f, 0.0f, 0.0f, 0.0000001f};




/****************** TEXTURES AND SAMPLERS*********************/



texture DiffuseMap : DiffuseMap
<
    string Name = "D.tga";
    string type = "2D";
>;

//could be anything here - ill,spec,normal,cube
texture EffectMap : DiffuseMap
<
    string Name = "I.tga";
    string type = "2D";
>;

texture EnvironmentMap 
< 
    string type = "2D"; 
    string name = "i.tga"; 
>;






//Diffuse Texture _D
sampler2D DiffuseSampler = sampler_state
{
    Texture   = <DiffuseMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//Effect Texture _I (could be anything here - ill,spec,normal,cube)
sampler2D EffectSampler = sampler_state
{
    Texture   = <EffectMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler EnvironmentSampler = sampler_state
{ 
    Texture = (EnvironmentMap);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

;





/************* DATA STRUCTS **************/

struct appdata {
    float4 Position	: POSITION;
    float4 UV0		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};



/*data passed to pixel shader*/
struct vertexOutput
{
    float4 Position    : POSITION;
    float2 TexCoord     : TEXCOORD0;
    float3 LightVec	    : TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldTangent	: TEXCOORD3;
    float3 WorldBinorm	: TEXCOORD4;
    float Fog : TEXCOORD5; 
    float4 WPos : TEXCOORD6;
    float clip : TEXCOORD7;
    
};


/*******Vertex Shader***************************/

vertexOutput mainVS(appdata IN)   
{
    
	vertexOutput OUT;
    
    //float4 tempPos = float4(IN.Position, 1);
    float4 worldSpacePos = mul(IN.Position, World);
    
    OUT.WorldNormal = normalize(mul(IN.Normal, WorldIT).xyz);
    OUT.WorldTangent = mul(IN.Tangent, WorldIT).xyz;
    OUT.WorldBinorm = mul(IN.Binormal, WorldIT).xyz;


    OUT.LightVec = normalize (LightSource - worldSpacePos );
    OUT.Position = mul(IN.Position, WorldViewProj);
    OUT.TexCoord  = IN.UV0; 
    
    OUT.WPos =   worldSpacePos;  
    
    // calculate FOG colour
    float4 cameraPos = mul( worldSpacePos, View );
    float fogstrength = cameraPos.z * FogColor.w;
    OUT.Fog = min(fogstrength,1.0); 
    
    // all shaders should send the clip value to the pixel shader (for refr/refl)                                                                     
    OUT.clip = dot(worldSpacePos, clipPlane);                                                                                        
                                           

    return OUT;
}


/****************Framgent Shader*****************/

float4 mainPS(vertexOutput IN) : COLOR
{
    
    // all shaders should receive the clip value                                                                
    clip(IN.clip);
    
    float4 diffuse = tex2D(DiffuseSampler,IN.TexCoord.xy);    //sample diffuse texture    
    float4 effectmap = tex2D(EffectSampler,IN.TexCoord.xy);    //sample specular map texture 
    float3 Ln = (IN.LightVec);
    float3 Nn = (IN.WorldNormal);
    float3 Tn = (IN.WorldTangent);
    float3 Bn = (IN.WorldBinorm);
    

   
     
    
    //normalized view vector for accurate specular highlights from LightSource
    float3 ViewVector  = (eyePos - IN.WPos);
    float3 Vn = normalize(ViewVector);
    float3 V = ViewVector;
    
    //reflection mapping for windows
    float3 G = normalize(2 * dot(Nn, Vn) * Nn - Vn);                // glance vector (view space)
    float3 Reflection = float3(-G.x, G.y, -G.z);
    float4 Env = tex2D(EnvironmentSampler,Reflection);
	//float4 Env = texCUBE(EnvironmentSampler,Reflection);  //no longer works without an actual cubemap
        
    //half vectors
    float3 Hn = normalize(Vn+Ln);
    float3 H = normalize(V+Ln);
    
    
   //calculate lighting Half Lamert scale and bias 
    float4 lighting = lit(pow(0.5*(dot(Ln,Nn))+0.5,1.0),dot(Hn,Nn),24);
    
    //calculate distance attenuation value based on eye position
	float dis = distance(IN.WPos,eyePos);
	//float atten = 1/(dis*.01*dis*.05+dis*.01);
	float atten = (1/(dis))*300;  //last value is multiplier, inverse square faloff
	atten = clamp(atten,0,1.2);  //second value controls how bright to let the highlights become

    
    //hero spec
    float herospec = pow(max(dot(Nn,H),0),36);
    float4 herospecContrib = herospec *effectmap;
    
    //give a slight boost to AmbiColor and clamp, prevents diffuse lighting from getting to dark on non-lit side
    AmbiColor = AmbiColor+0.1;
    AmbiColor = clamp(AmbiColor,0,0.1);

    
    
    //diffuse light contribution
    float4 diffuseContrib = diffuse * 1.2* lighting.y * (SurfColor+0.2) +(AmbiColor*diffuse);
    
    //specular light contribution
    float4 specContrib = lighting.z * effectmap*(SurfColor+0.1) ;
    
        
    
    float4 result =  diffuseContrib + (1*specContrib)+(0.6*herospecContrib)+(Env*0.4*effectmap.w);
    //float4 result = Env;
    result.w = 1.0 * diffuse.w;
    
    return float4((result.xyz*(1-IN.Fog))+(FogColor.xyz*IN.Fog),diffuse.w);
   
    
        
    
    
    
    return result;
}


/****** technique ********************************/

technique dx9textured
{
    pass P0
    {
        // lighting
        Lighting       = FALSE;
        FogEnable      = FALSE;

        // samplers
        //Sampler[0] = (LightmapSampler);
        //Sampler[1] = (DiffuseSampler);
        //Sampler[2] = (IllSpecSampler);
		//Sampler[3] = (NormalSampler);

        // shaders
        VertexShader = compile vs_2_0 mainVS();
        PixelShader  = compile ps_2_0 mainPS();
    }
}
