//Shader code by Mark Blosser
//WEBSITE:  www.hyrumark.com
//EMAIL:  mjblosser@gmail.com


//Description:  Shader for dynamic entities, combines a diffuse map with self-illuminated sprite map with mask.  
//Textures: 
//"texture_D.dds"      diffuse texture
//"texture_I.dd"       sprite atlas texture, black areas mask the animation


/************* UNTWEAKABLES **************/

float4x4 World      : WORLD;
float4x4 WorldViewProj : WorldViewProjection;
float4x4 WorldIT : WorldInverseTranspose;
float4x4 ViewInv : ViewInverse;
float4 eyePos : CameraPosition;
float Time: Time;
float sintime : SinTime;



/******TWEAKABLES***************************/


float SpriteRows : Power
<
    string UIName =  "Rows";
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 16.0;
    float UIStep = 1.0;
> = 4.0;

float SpriteColumns : Power
<
    string UIName =  "Columns";
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 16.0;
    float UIStep = 1.0;
> = 4.0;

float FramesPerSec : Power
<
    string UIName =  "Frames Per Second";
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 30.0;
    float UIStep = 0.5;
    
> = 15.0;

float Looptime : Power
<
    string UIName =  "Loop Time";
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 200.0;
    float UIStep = 1.0;
> = 100.0;

/******VALUES PULLED FROM FPSC - NON TWEAKABLE**********/

float4 AmbiColor : Ambient
<
    string UIName =  "Ambient Light Color";
> = {0.1f, 0.1f, 0.1f, 1.0f};

float4 SurfColor : Diffuse
<
    string UIName =  "Surface Color";
    string UIType = "Color";
> = {0.75f, 0.75f, 0.75f, 1.0f};

//float4 LightSource
//<
    //string UIType = "Fixed Light Source";
//> = {500.0f,500.0f, -100.0f, 1.0f};




/****************** TEXTURES AND SAMPLERS*********************/



texture DiffuseMap : DiffuseMap
<
    string Name = "D.tga";
    string type = "2D";
>;

//could be anything here - ill,spec,normal,cube
texture SpriteMap : DiffuseMap
<
    string Name = "I.tga";
    string type = "2D";
>;







//Diffuse Texture _D
sampler2D DiffuseSampler = sampler_state
{
    Texture   = <DiffuseMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//Effect Texture _I (could be anything here - ill,spec,normal,cube)
sampler2D SpriteSampler = sampler_state
{
    Texture   = <SpriteMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


;





/************* DATA STRUCTS **************/

struct appdata {
    float4 Position	: POSITION;
    float4 UV0		: TEXCOORD0;
    float4 UV1		: TEXCOORD1;
    float4 Normal	: NORMAL;
    
};



/*data passed to pixel shader*/
struct vertexOutput
{
    float4 Position    : POSITION;
    float2 TexCoord     : TEXCOORD0;
    float2 TexCoordLM   : TEXCOORD1;
    float3 WorldNormal	: TEXCOORD3;
    float4 WPos : TEXCOORD4;
    float2 atlasUV:TEXCOORD5;
};


/*******Vertex Shader***************************/

vertexOutput mainVS(appdata IN)   
{
    
	vertexOutput OUT;
    
    //float4 tempPos = float4(IN.Position, 1);
    float4 worldSpacePos = mul(IN.Position, World);
    
    OUT.WorldNormal = normalize(mul(IN.Normal, WorldIT).xyz);
    
    OUT.Position = mul(IN.Position, WorldViewProj);
    OUT.TexCoord  = IN.UV0; 
    OUT.TexCoordLM  = IN.UV1; 
    OUT.WPos =   worldSpacePos;   
    
    /****Calculate a set of texture coordinates to perform atlas (sprite) walking*********/
	float2 DimensionsXY = float2(SpriteRows,SpriteColumns); 
	float2 atlasUVtemp = IN.UV0; 
	
				
	float loopcounter  = floor(Time/Looptime); //increments by one every 50 seconds (or whatever "looptime" is)
	float offset ;  //initialize offset value used below
	
	offset = Looptime*loopcounter; //offset time value -increments every looptime
	
		 
	float speed =(Time*FramesPerSec) - (offset*FramesPerSec) ;
	
	
	float index = floor( speed);		//floor of speed
	float rowCount = floor( (index / DimensionsXY.y) );		//floor of (speed / Ydimension.g)
	float2 offsetVector = float2(index, rowCount);
	float2 atlas = (1.0 / DimensionsXY) ;
	float2 move = (offsetVector + atlasUVtemp);
	
	OUT.atlasUV = (atlas.xy *move);
	/*******************************************************************************************/   
	
    
                                                                        
                                           

    return OUT;
}


/****************Framgent Shader*****************/

float4 mainPS(vertexOutput IN) : COLOR
{
    
    //Sample Textures
    float4 diffuse = tex2D(DiffuseSampler,IN.TexCoord.xy);    //sample diffuse texture    
    float4 atlasill = tex2D(SpriteSampler,IN.atlasUV.xy);    //sample specular map texture
    
    //float3 Nn = (IN.WorldNormal);
    SurfColor = clamp(SurfColor,0.1,0.9); //clamp SurfColor to prevent zero lighting and overly bright lighting
    //AmbiColor = clamp(AmbiColor,0,0.5);
    
    float4 diffuseContrib = diffuse * (SurfColor) +(AmbiColor*diffuse);
    
    
    float4 spriteContrib  = (atlasill)*5*diffuse-0.1;//adding in a bit of contrast here
    
    spriteContrib = clamp(spriteContrib,0,1);  //clamp value between 0,1 to keep from going BELOW zero and pulling brightness from final lighting
   
    float4 result =   diffuseContrib + spriteContrib;
    
    
    return result;
}


/****** technique ********************************/

technique dx9textured
{
    pass P0
    {
        // lighting
        Lighting       = FALSE;
        FogEnable      = FALSE;

        // samplers
        //Sampler[0] = (LightmapSampler);
        //Sampler[1] = (DiffuseSampler);
        //Sampler[2] = (IllSpecSampler);
		//Sampler[3] = (NormalSampler);

        // shaders
        VertexShader = compile vs_2_0 mainVS();
        PixelShader  = compile ps_2_0 mainPS();
    }
}
